import time
from easy_vk.exceptions.exceptions import raise_exception, Server
from typing import Optional, Tuple, List, Dict, Any
from ctypes import c_longdouble


def preprocess_parameter(parameter):
    if isinstance(parameter, dict) or isinstance(parameter, str):
        return parameter
    if isinstance(parameter, int) or isinstance(parameter, float):
        return str(parameter)
    if isinstance(parameter, list):
        parameter = [preprocess_parameter(p) for p in parameter]
        parameter = ','.join(parameter)
        return parameter

    if hasattr(parameter, '__dict__'):
        if hasattr(parameter, 'value'):
            return parameter.value
        elif hasattr(parameter, 'Config'):
            return parameter.json(exclude_none=True)
        else:
            ValueError(f'Unknown parameter type passed ({parameter}).\n'
                       f'Parameters can be only builtin types or objects, defined in easy_vk.types.objects.')


def unpack_response(response, response_type):
    if response_type in (int, bool, str, float):
        return response_type(response)
    elif hasattr(response_type, '__origin__'):
        return [unpack_response(r, response_type.__args__[0]) for r in response]
    elif hasattr(response_type, '__fields__'):
        return response_type(**response)
    else:
        pass


class BaseCategory:
    def __init__(self, session, access_token: str, v: str, last_call_timer: c_longdouble, delay: float,
                 auto_retry: bool,
                 max_retries: int,
                 timeout: float):
        """
        Base api category class

        :param session: api session
        :param access_token: api access_token
        :param v: api version
        :param delay: delay between api calls
        :param auto_retry: enables auto retry in calling methods if specific errors occurred
            e.g. easy_vk.exceptions.exceptions.Server
        :param max_retries: maximum value of retries. Works only if auto_retry is True
        :param timeout: time to sleep between retries. Works only if auto_retry is True
        """

        self._last_call_timer = last_call_timer
        self._session = session
        self._access_token = access_token
        self._v = v
        self._delay = delay
        self._auto_retry = auto_retry
        self._max_retries = max_retries
        self._timeout = timeout

    def _call(self, method_name: str, method_parameters: Dict[str, Any], param_aliases: Optional[List[Tuple[str, str]]],
              response_type, retries_count: int = 0):
        """
        Call method "method_name" with parameters and return json or object response

        :param method_name: full name of the method.  e.g. friends.get
        :param method_parameters: locals, containing method parameters
        :param param_aliases: Optional[List[Tuple[str, str]]] e.g [('type_', 'type'), ('global_', 'global')]
        :param response_type: response object which method should return
            e.g easy_vk.types.responses.FriendsGetResponse
        :param retries_count: current retries counter
        """
        api_url = f'https://api.vk.com/method/{method_name}'

        if param_aliases:
            for name, alias in param_aliases:
                method_parameters[alias] = method_parameters.pop(name, None)

        params = {parameter: value for parameter, value in method_parameters.items() if value is not None}
        params = {p: preprocess_parameter(params[p]) for p in params}
        params['access_token'] = self._access_token
        params['v'] = self._v

        delay = self._delay - (time.time() - self._last_call_timer.value)
        if delay > 0:
            time.sleep(delay)

        try:
            # post request type to have larger size requests
            response = self._session.post(url=api_url, params=params).json()

            if 'response' in response:
                response = response['response']

            # error
            else:
                error_code = response['error']['error_code']
                error_message = response['error']['error_msg']
                raise_exception(error_code, error_message)

        except Server as e:
            if self._auto_retry and retries_count < self._max_retries:
                time.sleep(self._timeout)
                response = self._call(method_name, method_parameters,
                                      param_aliases, response_type, retries_count=retries_count + 1)
            else:
                raise e

        self._last_call_timer.value = time.time()
        response = unpack_response(response, response_type)

        return response



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Fave(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def add_article(self, url: str) -> responses.BaseOk:
        """
        :param url:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.addArticle'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_link(self, link: str) -> responses.BaseOk:
        """
        Adds a link to user faves.
        
        :param link: Link URL.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.addLink'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_page(self, user_id: Optional[int] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        :param user_id:
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.addPage'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_post(self, owner_id: int, id_: int, access_key: Optional[str] = None) -> responses.BaseOk:
        """
        :param owner_id:
        :param id_:
        :param access_key:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.addPost'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_product(self, owner_id: int, id_: int, access_key: Optional[str] = None) -> responses.BaseOk:
        """
        :param owner_id:
        :param id_:
        :param access_key:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.addProduct'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_tag(self, name: Optional[str] = None, position: Optional[str] = None) -> responses.FaveAddTag:
        """
        :param name:
        :param position:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.addTag'
        response_type = responses.FaveAddTag
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def add_video(self, owner_id: int, id_: int, access_key: Optional[str] = None) -> responses.BaseOk:
        """
        :param owner_id:
        :param id_:
        :param access_key:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.addVideo'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit_tag(self, id_: int, name: str) -> responses.BaseOk:
        """
        :param id_:
        :param name:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.editTag'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get(self, extended: None = None, item_type: Optional[str] = None, tag_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, fields: Optional[str] = None, is_from_snackbar: Optional[bool] = None) -> responses.FaveGet: ...
    @overload
    def get(self, extended: bool, item_type: Optional[str] = None, tag_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, fields: Optional[str] = None, is_from_snackbar: Optional[bool] = None) -> responses.FaveGetExtended: ...
    def get(self, extended: Optional[bool] = None, item_type: Optional[str] = None, tag_id: Optional[int] = None, offset: Optional[int] = None, count: Optional[int] = None, fields: Optional[str] = None, is_from_snackbar: Optional[bool] = None):
        """
        :param extended: '1' — to return additional 'wall', 'profiles', and 'groups' fields. By default: '0'.
        :param item_type:
        :param tag_id: Tag ID.
        :param offset: Offset needed to return a specific subset of users.
        :param count: Number of users to return.
        :param fields:
        :param is_from_snackbar:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.get'
        if not extended:
            response_type = responses.FaveGet
        if extended:
            response_type = responses.FaveGetExtended
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_pages(self, offset: Optional[int] = None, count: Optional[int] = None, type_: Optional[str] = None, fields: Optional[List[Union[objects.BaseUserGroupFields, str]]] = None, tag_id: Optional[int] = None) -> responses.FaveGetPages:
        """
        :param offset:
        :param count:
        :param type_:
        :param fields:
        :param tag_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('type_', 'type')]
        method_name = 'fave.getPages'
        response_type = responses.FaveGetPages
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_tags(self) -> responses.FaveGetTags:
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.getTags'
        response_type = responses.FaveGetTags
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def mark_seen(self) -> responses.BaseBool:
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.markSeen'
        response_type = responses.BaseBool
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_article(self, owner_id: int, article_id: int) -> responses.BaseBool:
        """
        :param owner_id:
        :param article_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.removeArticle'
        response_type = responses.BaseBool
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_link(self, link_id: Optional[str] = None, link: Optional[str] = None) -> responses.BaseOk:
        """
        Removes link from the user's faves.
        
        :param link_id: Link ID (can be obtained by [vk.com/dev/faves.getLinks|faves.getLinks] method).
        :param link: Link URL
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.removeLink'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_page(self, user_id: Optional[int] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        :param user_id:
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.removePage'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_post(self, owner_id: int, id_: int) -> responses.BaseOk:
        """
        :param owner_id:
        :param id_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.removePost'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_product(self, owner_id: int, id_: int) -> responses.BaseOk:
        """
        :param owner_id:
        :param id_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.removeProduct'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_tag(self, id_: int) -> responses.BaseOk:
        """
        :param id_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('id_', 'id')]
        method_name = 'fave.removeTag'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def reorder_tags(self, ids: List[int]) -> responses.BaseOk:
        """
        :param ids:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.reorderTags'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def set_page_tags(self, user_id: Optional[int] = None, group_id: Optional[int] = None, tag_ids: Optional[List[int]] = None) -> responses.BaseOk:
        """
        :param user_id:
        :param group_id:
        :param tag_ids:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.setPageTags'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def set_tags(self, item_type: Optional[str] = None, item_owner_id: Optional[int] = None, item_id: Optional[int] = None, tag_ids: Optional[List[int]] = None, link_id: Optional[str] = None, link_url: Optional[str] = None) -> responses.BaseOk:
        """
        :param item_type:
        :param item_owner_id:
        :param item_id:
        :param tag_ids:
        :param link_id:
        :param link_url:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.setTags'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def track_page_interaction(self, user_id: Optional[int] = None, group_id: Optional[int] = None) -> responses.BaseOk:
        """
        :param user_id:
        :param group_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'fave.trackPageInteraction'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Docs(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def add(self, owner_id: int, doc_id: int, access_key: Optional[str] = None) -> responses.DocsAdd:
        """
        Copies a document to a user's or community's document list.
        
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param access_key: Access key. This parameter is required if 'access_key' was returned with the document's data.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.add'
        response_type = responses.DocsAdd
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete(self, owner_id: int, doc_id: int) -> responses.BaseOk:
        """
        Deletes a user or community document.
        
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.delete'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit(self, owner_id: int, doc_id: int, title: Optional[str] = None, tags: Optional[List[str]] = None) -> responses.BaseOk:
        """
        Edits a document.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param title: Document title.
        :param tags: Document tags.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.edit'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get(self, count: Optional[int] = None, offset: Optional[int] = None, type_: Optional[int] = None, owner_id: Optional[int] = None, return_tags: Optional[bool] = None) -> responses.DocsGet:
        """
        Returns detailed information about user or community documents.
        
        :param count: Number of documents to return. By default, all documents.
        :param offset: Offset needed to return a specific subset of documents.
        :param type_:
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('type_', 'type')]
        method_name = 'docs.get'
        response_type = responses.DocsGet
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_by_id(self, docs: List[str], return_tags: Optional[bool] = None) -> responses.DocsGetById:
        """
        Returns information about documents by their IDs.
        
        :param docs: Document IDs. Example: , "66748_91488,66748_91455",
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.getById'
        response_type = responses.DocsGetById
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_messages_upload_server(self, type_: Optional[str] = None, peer_id: Optional[int] = None) -> responses.BaseGetUploadServer:
        """
        Returns the server address for document upload.
        
        :param type_: Document type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('type_', 'type')]
        method_name = 'docs.getMessagesUploadServer'
        response_type = responses.BaseGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_types(self, owner_id: int) -> responses.DocsGetTypes:
        """
        Returns documents types available for current user.
        
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.getTypes'
        response_type = responses.DocsGetTypes
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_upload_server(self, group_id: Optional[int] = None) -> responses.DocsGetUploadServer:
        """
        Returns the server address for document upload.
        
        :param group_id: Community ID (if the document will be uploaded to the community).
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.getUploadServer'
        response_type = responses.DocsGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_wall_upload_server(self, group_id: Optional[int] = None) -> responses.BaseGetUploadServer:
        """
        Returns the server address for document upload onto a user's or community's wall.
        
        :param group_id: Community ID (if the document will be uploaded to the community).
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.getWallUploadServer'
        response_type = responses.BaseGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def save(self, file: str, title: Optional[str] = None, tags: Optional[str] = None, return_tags: Optional[bool] = None) -> responses.DocsSave:
        """
        Saves a document after [vk.com/dev/upload_files_2|uploading it to a server].
        
        :param file: This parameter is returned when the file is [vk.com/dev/upload_files_2|uploaded to the server].
        :param title: Document title.
        :param tags: Document tags.
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.save'
        response_type = responses.DocsSave
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def search(self, q: str, search_own: Optional[bool] = None, count: Optional[int] = None, offset: Optional[int] = None, return_tags: Optional[bool] = None) -> responses.DocsSearch:
        """
        Returns a list of documents matching the search criteria.
        
        :param q: Search query string.
        :param search_own:
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.
        :param return_tags:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'docs.search'
        response_type = responses.DocsSearch
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Ads(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def add_office_users(self, account_id: int, data: str) -> responses.AdsAddOfficeUsers:
        """
        Adds managers and/or supervisors to advertising account.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe added managers. Description of 'user_specification' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.addOfficeUsers'
        response_type = responses.AdsAddOfficeUsers
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def check_link(self, account_id: int, link_type: str, link_url: str, campaign_id: Optional[int] = None) -> responses.AdsCheckLink:
        """
        Allows to check the ad link.
        
        :param account_id: Advertising account ID.
        :param link_type: Object type: *'community' — community,, *'post' — community post,, *'application' — VK application,, *'video' — video,, *'site' — external site.
        :param link_url: Object URL.
        :param campaign_id: Campaign ID
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.checkLink'
        response_type = responses.AdsCheckLink
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_ads(self, account_id: int, data: str) -> responses.AdsCreateAds:
        """
        Creates ads.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created ads. Description of 'ad_specification' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.createAds'
        response_type = responses.AdsCreateAds
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_campaigns(self, account_id: int, data: str) -> responses.AdsCreateCampaigns:
        """
        Creates advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'campaign_specification' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.createCampaigns'
        response_type = responses.AdsCreateCampaigns
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_clients(self, account_id: int, data: str) -> responses.AdsCreateClients:
        """
        Creates clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe created campaigns. Description of 'client_specification' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.createClients'
        response_type = responses.AdsCreateClients
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create_target_group(self, account_id: int, name: str, lifetime: int, client_id: Optional[int] = None, target_pixel_id: Optional[int] = None, target_pixel_rules: Optional[str] = None) -> responses.AdsCreateTargetGroup:
        """
        Creates a group to re-target ads for users who visited advertiser's site (viewed information about the product, registered, etc.).
        
        :param account_id: Advertising account ID.
        :param name: Name of the target group — a string up to 64 characters long.
        :param lifetime: 'For groups with auditory created with pixel code only.', , Number of days after that users will be automatically removed from the group.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param target_pixel_id:
        :param target_pixel_rules:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.createTargetGroup'
        response_type = responses.AdsCreateTargetGroup
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_ads(self, account_id: int, ids: str) -> responses.AdsDeleteAds:
        """
        Archives ads.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with ad IDs.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.deleteAds'
        response_type = responses.AdsDeleteAds
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_campaigns(self, account_id: int, ids: str) -> responses.AdsDeleteCampaigns:
        """
        Archives advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted campaigns.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.deleteCampaigns'
        response_type = responses.AdsDeleteCampaigns
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_clients(self, account_id: int, ids: str) -> responses.AdsDeleteClients:
        """
        Archives clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted clients.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.deleteClients'
        response_type = responses.AdsDeleteClients
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_target_group(self, account_id: int, target_group_id: int, client_id: Optional[int] = None) -> responses.BaseOk:
        """
        Deletes a retarget group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.deleteTargetGroup'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_accounts(self) -> responses.AdsGetAccounts:
        """
        Returns a list of advertising accounts.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getAccounts'
        response_type = responses.AdsGetAccounts
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_ads(self, account_id: int, ad_ids: Optional[str] = None, campaign_ids: Optional[str] = None, client_id: Optional[int] = None, include_deleted: Optional[bool] = None, only_deleted: Optional[bool] = None, limit: Optional[int] = None, offset: Optional[int] = None) -> responses.AdsGetAds:
        """
        Returns number of ads.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'Available and required for advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param only_deleted: Flag that specifies whether to show only archived ads: *0 — show all ads,, *1 — show only archived ads. Available when include_deleted flag is *1
        :param limit: Limit of number of returned ads. Used only if ad_ids parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getAds'
        response_type = responses.AdsGetAds
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_ads_layout(self, account_id: int, ad_ids: Optional[str] = None, campaign_ids: Optional[str] = None, client_id: Optional[int] = None, include_deleted: Optional[bool] = None, limit: Optional[int] = None, offset: Optional[int] = None) -> responses.AdsGetAdsLayout:
        """
        Returns descriptions of ad layouts.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset. Used in the same cases as 'limit' parameter.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getAdsLayout'
        response_type = responses.AdsGetAdsLayout
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_ads_targeting(self, account_id: int, ad_ids: Optional[str] = None, campaign_ids: Optional[str] = None, client_id: Optional[int] = None, include_deleted: Optional[bool] = None, limit: Optional[int] = None, offset: Optional[int] = None) -> responses.AdsGetAdsTargeting:
        """
        Returns ad targeting parameters.
        
        :param account_id: Advertising account ID.
        :param ad_ids: Filter by ads. Serialized JSON array with ad IDs. If the parameter is null, all ads will be shown.
        :param campaign_ids: Filter by advertising campaigns. Serialized JSON array with campaign IDs. If the parameter is null, ads of all campaigns will be shown.
        :param client_id: 'For advertising agencies.' ID of the client ads are retrieved from.
        :param include_deleted: flag that specifies whether archived ads shall be shown: *0 — show only active ads,, *1 — show all ads.
        :param limit: Limit of number of returned ads. Used only if 'ad_ids' parameter is null, and 'campaign_ids' parameter contains ID of only one campaign.
        :param offset: Offset needed to return a specific subset of results.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getAdsTargeting'
        response_type = responses.AdsGetAdsTargeting
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_budget(self, account_id: int) -> responses.AdsGetBudget:
        """
        Returns current budget of the advertising account.
        
        :param account_id: Advertising account ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getBudget'
        response_type = responses.AdsGetBudget
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_campaigns(self, account_id: int, client_id: Optional[int] = None, include_deleted: Optional[bool] = None, campaign_ids: Optional[str] = None, fields: Optional[List[str]] = None) -> responses.AdsGetCampaigns:
        """
        Returns a list of campaigns in an advertising account.
        
        :param account_id: Advertising account ID.
        :param client_id: 'For advertising agencies'. ID of the client advertising campaigns are retrieved from.
        :param include_deleted: Flag that specifies whether archived ads shall be shown. *0 — show only active campaigns,, *1 — show all campaigns.
        :param campaign_ids: Filter of advertising campaigns to show. Serialized JSON array with campaign IDs. Only campaigns that exist in 'campaign_ids' and belong to the specified advertising account will be shown. If the parameter is null, all campaigns will be shown.
        :param fields:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getCampaigns'
        response_type = responses.AdsGetCampaigns
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_categories(self, lang: Optional[str] = None) -> responses.AdsGetCategories:
        """
        Returns a list of possible ad categories.
        
        :param lang: Language. The full list of supported languages is [vk.com/dev/api_requests|here].
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getCategories'
        response_type = responses.AdsGetCategories
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_clients(self, account_id: int) -> responses.AdsGetClients:
        """
        Returns a list of advertising agency's clients.
        
        :param account_id: Advertising account ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getClients'
        response_type = responses.AdsGetClients
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_demographics(self, account_id: int, ids_type: str, ids: str, period: str, date_from: str, date_to: str) -> responses.AdsGetDemographics:
        """
        Returns demographics for ads or campaigns.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getDemographics'
        response_type = responses.AdsGetDemographics
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_flood_stats(self, account_id: int) -> responses.AdsGetFloodStats:
        """
        Returns information about current state of a counter — number of remaining runs of methods and time to the next counter nulling in seconds.
        
        :param account_id: Advertising account ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getFloodStats'
        response_type = responses.AdsGetFloodStats
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_lookalike_requests(self, account_id: int, client_id: Optional[int] = None, requests_ids: Optional[str] = None, offset: Optional[int] = None, limit: Optional[int] = None, sort_by: Optional[str] = None) -> responses.AdsGetLookalikeRequests:
        """
        :param account_id:
        :param client_id:
        :param requests_ids:
        :param offset:
        :param limit:
        :param sort_by:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getLookalikeRequests'
        response_type = responses.AdsGetLookalikeRequests
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_musicians(self, artist_name: str) -> responses.AdsGetMusicians:
        """
        :param artist_name:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getMusicians'
        response_type = responses.AdsGetMusicians
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_office_users(self, account_id: int) -> responses.AdsGetOfficeUsers:
        """
        Returns a list of managers and supervisors of advertising account.
        
        :param account_id: Advertising account ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getOfficeUsers'
        response_type = responses.AdsGetOfficeUsers
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_posts_reach(self, account_id: int, ids_type: str, ids: str) -> responses.AdsGetPostsReach:
        """
        Returns detailed statistics of promoted posts reach from campaigns and ads.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns.
        :param ids: IDs requested ads or campaigns, separated with a comma, depending on the value set in 'ids_type'. Maximum 100 objects.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getPostsReach'
        response_type = responses.AdsGetPostsReach
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_rejection_reason(self, account_id: int, ad_id: int) -> responses.AdsGetRejectionReason:
        """
        Returns a reason of ad rejection for pre-moderation.
        
        :param account_id: Advertising account ID.
        :param ad_id: Ad ID.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getRejectionReason'
        response_type = responses.AdsGetRejectionReason
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_statistics(self, account_id: int, ids_type: str, ids: str, period: str, date_from: str, date_to: str, stats_fields: Optional[List[str]] = None) -> responses.AdsGetStatistics:
        """
        Returns statistics of performance indicators for ads, campaigns, clients or the whole account.
        
        :param account_id: Advertising account ID.
        :param ids_type: Type of requested objects listed in 'ids' parameter: *ad — ads,, *campaign — campaigns,, *client — clients,, *office — account.
        :param ids: IDs requested ads, campaigns, clients or account, separated with a comma, depending on the value set in 'ids_type'. Maximum 2000 objects.
        :param period: Data grouping by dates: *day — statistics by days,, *month — statistics by months,, *overall — overall statistics. 'date_from' and 'date_to' parameters set temporary limits.
        :param date_from: Date to show statistics from. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — day it was created on,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — month it was created in,, *overall: 0.
        :param date_to: Date to show statistics to. For different value of 'period' different date format is used: *day: YYYY-MM-DD, example: 2011-09-27 — September 27, 2011, **0 — current day,, *month: YYYY-MM, example: 2011-09 — September 2011, **0 — current month,, *overall: 0.
        :param stats_fields: Additional fields to add to statistics
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getStatistics'
        response_type = responses.AdsGetStatistics
        return self._call(method_name, method_parameters, param_aliases, response_type)

    @overload
    def get_suggestions(self, section: Literal['countries', 'positions', 'interest_categories', 'religions', 'user_devices', 'user_os', 'user_browsers'], ids: Optional[str] = None, q: Optional[str] = None, country: Optional[int] = None, cities: Optional[str] = None, lang: Optional[str] = None) -> responses.AdsGetSuggestions: ...
    @overload
    def get_suggestions(self, section: Literal['regions'], ids: Optional[str] = None, q: Optional[str] = None, country: Optional[int] = None, cities: Optional[str] = None, lang: Optional[str] = None) -> responses.AdsGetSuggestionsRegions: ...
    @overload
    def get_suggestions(self, section: Literal['cities', 'districts', 'streets'], ids: Optional[str] = None, q: Optional[str] = None, country: Optional[int] = None, cities: Optional[str] = None, lang: Optional[str] = None) -> responses.AdsGetSuggestionsCities: ...
    @overload
    def get_suggestions(self, section: Literal['schools'], ids: Optional[str] = None, q: Optional[str] = None, country: Optional[int] = None, cities: Optional[str] = None, lang: Optional[str] = None) -> responses.AdsGetSuggestionsSchools: ...
    def get_suggestions(self, section: str, ids: Optional[str] = None, q: Optional[str] = None, country: Optional[int] = None, cities: Optional[str] = None, lang: Optional[str] = None):
        """
        Returns a set of auto-suggestions for various targeting parameters.
        
        :param section: Section, suggestions are retrieved in. Available values: *countries — request of a list of countries. If q is not set or blank, a short list of countries is shown. Otherwise, a full list of countries is shown. *regions — requested list of regions. 'country' parameter is required. *cities — requested list of cities. 'country' parameter is required. *districts — requested list of districts. 'cities' parameter is required. *stations — requested list of subway stations. 'cities' parameter is required. *streets — requested list of streets. 'cities' parameter is required. *schools — requested list of educational organizations. 'cities' parameter is required. *interests — requested list of interests. *positions — requested list of positions (professions). *group_types — requested list of group types. *religions — requested list of religious commitments. *browsers — requested list of browsers and mobile devices.
        :param ids: Objects IDs separated by commas. If the parameter is passed, 'q, country, cities' should not be passed.
        :param q: Filter-line of the request (for countries, regions, cities, streets, schools, interests, positions).
        :param country: ID of the country objects are searched in.
        :param cities: IDs of cities where objects are searched in, separated with a comma.
        :param lang: Language of the returned string values. Supported languages: *ru — Russian,, *ua — Ukrainian,, *en — English.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getSuggestions'
        if section in ['countries', 'positions', 'interest_categories', 'religions', 'user_devices', 'user_os', 'user_browsers']:
            response_type = responses.AdsGetSuggestions
        if section in ['regions']:
            response_type = responses.AdsGetSuggestionsRegions
        if section in ['cities', 'districts', 'streets']:
            response_type = responses.AdsGetSuggestionsCities
        if section in ['schools']:
            response_type = responses.AdsGetSuggestionsSchools
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_target_groups(self, account_id: int, client_id: Optional[int] = None, extended: Optional[bool] = None) -> responses.AdsGetTargetGroups:
        """
        Returns a list of target groups.
        
        :param account_id: Advertising account ID.
        :param client_id: 'Only for advertising agencies.', ID of the client with the advertising account where the group will be created.
        :param extended: '1' — to return pixel code.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getTargetGroups'
        response_type = responses.AdsGetTargetGroups
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_targeting_stats(self, account_id: int, link_url: str, client_id: Optional[int] = None, criteria: Optional[str] = None, ad_id: Optional[int] = None, ad_format: Optional[int] = None, ad_platform: Optional[str] = None, ad_platform_no_wall: Optional[str] = None, ad_platform_no_ad_network: Optional[str] = None, link_domain: Optional[str] = None, need_precise: Optional[bool] = None) -> responses.AdsGetTargetingStats:
        """
        Returns the size of targeting audience, and also recommended values for CPC and CPM.
        
        :param account_id: Advertising account ID.
        :param link_url: URL for the advertised object.
        :param client_id:
        :param criteria: Serialized JSON object that describes targeting parameters. Description of 'criteria' object see below.
        :param ad_id: ID of an ad which targeting parameters shall be analyzed.
        :param ad_format: Ad format. Possible values: *'1' — image and text,, *'2' — big image,, *'3' — exclusive format,, *'4' — community, square image,, *'7' — special app format,, *'8' — special community format,, *'9' — post in community,, *'10' — app board.
        :param ad_platform: Platforms to use for ad showing. Possible values: (for 'ad_format' = '1'), *'0' — VK and partner sites,, *'1' — VK only. (for 'ad_format' = '9'), *'all' — all platforms,, *'desktop' — desktop version,, *'mobile' — mobile version and apps.
        :param ad_platform_no_wall:
        :param ad_platform_no_ad_network:
        :param link_domain: Domain of the advertised object.
        :param need_precise: Additionally return recommended cpc and cpm to reach 5,10..95 percents of audience.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getTargetingStats'
        response_type = responses.AdsGetTargetingStats
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_upload_url(self, ad_format: int, icon: Optional[int] = None) -> responses.AdsGetUploadUrl:
        """
        Returns URL to upload an ad photo to.
        
        :param ad_format: Ad format: *1 — image and text,, *2 — big image,, *3 — exclusive format,, *4 — community, square image,, *7 — special app format.
        :param icon:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getUploadURL'
        response_type = responses.AdsGetUploadUrl
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_video_upload_url(self) -> responses.AdsGetVideoUploadUrl:
        """
        Returns URL to upload an ad video to.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.getVideoUploadURL'
        response_type = responses.AdsGetVideoUploadUrl
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def import_target_contacts(self, account_id: int, target_group_id: int, contacts: str, client_id: Optional[int] = None) -> responses.AdsImportTargetContacts:
        """
        Imports a list of advertiser's contacts to count VK registered users against the target group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Target group ID.
        :param contacts: List of phone numbers, emails or user IDs separated with a comma.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.importTargetContacts'
        response_type = responses.AdsImportTargetContacts
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def remove_office_users(self, account_id: int, ids: str) -> responses.AdsRemoveOfficeUsers:
        """
        Removes managers and/or supervisors from advertising account.
        
        :param account_id: Advertising account ID.
        :param ids: Serialized JSON array with IDs of deleted managers.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.removeOfficeUsers'
        response_type = responses.AdsRemoveOfficeUsers
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def update_ads(self, account_id: int, data: str) -> responses.AdsUpdateAds:
        """
        Edits ads.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in ads. Description of 'ad_edit_specification' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.updateAds'
        response_type = responses.AdsUpdateAds
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def update_campaigns(self, account_id: int, data: str) -> responses.AdsUpdateCampaigns:
        """
        Edits advertising campaigns.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in campaigns. Description of 'campaign_mod' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.updateCampaigns'
        response_type = responses.AdsUpdateCampaigns
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def update_clients(self, account_id: int, data: str) -> responses.AdsUpdateClients:
        """
        Edits clients of an advertising agency.
        
        :param account_id: Advertising account ID.
        :param data: Serialized JSON array of objects that describe changes in clients. Description of 'client_mod' objects see below.
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.updateClients'
        response_type = responses.AdsUpdateClients
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def update_target_group(self, account_id: int, target_group_id: int, name: str, lifetime: int, client_id: Optional[int] = None, domain: Optional[str] = None, target_pixel_id: Optional[int] = None, target_pixel_rules: Optional[str] = None) -> responses.BaseOk:
        """
        Edits a retarget group.
        
        :param account_id: Advertising account ID.
        :param target_group_id: Group ID.
        :param name: New name of the target group — a string up to 64 characters long.
        :param lifetime: 'Only for the groups that get audience from sites with user accounting code.', Time in days when users added to a retarget group will be automatically excluded from it. '0' - automatic exclusion is off.
        :param client_id: 'Only for advertising agencies.' , ID of the client with the advertising account where the group will be created.
        :param domain: Domain of the site where user accounting code will be placed.
        :param target_pixel_id:
        :param target_pixel_rules:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'ads.updateTargetGroup'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Polls(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def add_vote(self, poll_id: int, answer_ids: List[int], owner_id: Optional[int] = None, is_board: Optional[bool] = None) -> responses.PollsAddVote:
        """
        Adds the current user's vote to the selected answer in the poll.
        
        :param poll_id: Poll ID.
        :param answer_ids:
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.addVote'
        response_type = responses.PollsAddVote
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def create(self, question: Optional[str] = None, is_anonymous: Optional[bool] = None, is_multiple: Optional[bool] = None, end_date: Optional[int] = None, owner_id: Optional[int] = None, add_answers: Optional[str] = None, photo_id: Optional[int] = None, background_id: Optional[str] = None, disable_unvote: Optional[bool] = None) -> responses.PollsCreate:
        """
        Creates polls that can be attached to the users' or communities' posts.
        
        :param question: question text
        :param is_anonymous: '1' – anonymous poll, participants list is hidden,, '0' – public poll, participants list is available,, Default value is '0'.
        :param is_multiple:
        :param end_date:
        :param owner_id: If a poll will be added to a communty it is required to send a negative group identifier. Current user by default.
        :param add_answers: available answers list, for example: " ["yes","no","maybe"]", There can be from 1 to 10 answers.
        :param photo_id:
        :param background_id:
        :param disable_unvote:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.create'
        response_type = responses.PollsCreate
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def delete_vote(self, poll_id: int, answer_id: int, owner_id: Optional[int] = None, is_board: Optional[bool] = None) -> responses.PollsDeleteVote:
        """
        Deletes the current user's vote from the selected answer in the poll.
        
        :param poll_id: Poll ID.
        :param answer_id: Answer ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.deleteVote'
        response_type = responses.PollsDeleteVote
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def edit(self, poll_id: int, owner_id: Optional[int] = None, question: Optional[str] = None, add_answers: Optional[str] = None, edit_answers: Optional[str] = None, delete_answers: Optional[str] = None, end_date: Optional[int] = None, photo_id: Optional[int] = None, background_id: Optional[str] = None) -> responses.BaseOk:
        """
        Edits created polls
        
        :param poll_id: edited poll's id
        :param owner_id: poll owner id
        :param question: new question text
        :param add_answers: answers list, for example: , "["yes","no","maybe"]"
        :param edit_answers: object containing answers that need to be edited,, key – answer id, value – new answer text. Example: {"382967099":"option1", "382967103":"option2"}"
        :param delete_answers: list of answer ids to be deleted. For example: "[382967099, 382967103]"
        :param end_date:
        :param photo_id:
        :param background_id:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.edit'
        response_type = responses.BaseOk
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_by_id(self, poll_id: int, owner_id: Optional[int] = None, is_board: Optional[bool] = None, extended: Optional[bool] = None, friends_count: Optional[int] = None, fields: Optional[List[str]] = None, name_case: Optional[str] = None) -> responses.PollsGetById:
        """
        Returns detailed information about a poll by its ID.
        
        :param poll_id: Poll ID.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board: '1' – poll is in a board, '0' – poll is on a wall. '0' by default.
        :param extended:
        :param friends_count:
        :param fields:
        :param name_case:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.getById'
        response_type = responses.PollsGetById
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_voters(self, poll_id: int, answer_ids: List[int], owner_id: Optional[int] = None, is_board: Optional[bool] = None, friends_only: Optional[bool] = None, offset: Optional[int] = None, count: Optional[int] = None, fields: Optional[List[Union[objects.UsersFields, str]]] = None, name_case: Optional[str] = None) -> responses.PollsGetVoters:
        """
        Returns a list of IDs of users who selected specific answers in the poll.
        
        :param poll_id: Poll ID.
        :param answer_ids: Answer IDs.
        :param owner_id: ID of the user or community that owns the poll. Use a negative value to designate a community ID.
        :param is_board:
        :param friends_only: '1' — to return only current user's friends, '0' — to return all users (default),
        :param offset: Offset needed to return a specific subset of voters. '0' — (default)
        :param count: Number of user IDs to return (if the 'friends_only' parameter is not set, maximum '1000', otherwise '10'). '100' — (default)
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate (birthdate)', 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online', 'counters'.
        :param name_case: Case for declension of user name and surname: , 'nom' — nominative (default) , 'gen' — genitive , 'dat' — dative , 'acc' — accusative , 'ins' — instrumental , 'abl' — prepositional
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'polls.getVoters'
        response_type = responses.PollsGetVoters
        return self._call(method_name, method_parameters, param_aliases, response_type)



# This file was autogenerated from vk-api json schema

from typing import List, Union, Optional, overload
from easy_vk.types import objects
from easy_vk.types import responses
from easy_vk.api_category import BaseCategory
try:
    from typing import Literal
except Exception:
    from typing_extensions import Literal


class Photos(BaseCategory):
    def __init__(self, session, access_token: str, v: str, last_call_timer, delay: float, auto_retry: bool, max_retries: int, timeout: float):
        super().__init__(session, access_token, v, last_call_timer, delay, auto_retry, max_retries, timeout)

    def get_messages_upload_server(self, peer_id: Optional[int] = None) -> responses.PhotosGetMessagesUploadServer:
        """
        Returns the server address for photo upload in a private message for a user.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'photos.getMessagesUploadServer'
        response_type = responses.PhotosGetMessagesUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def get_owner_cover_photo_upload_server(self, group_id: int, crop_x: Optional[int] = None, crop_y: Optional[int] = None, crop_x2: Optional[int] = None, crop_y2: Optional[int] = None) -> responses.BaseGetUploadServer:
        """
        Returns the server address for owner cover upload.
        
        :param group_id: ID of community that owns the album (if the photo will be uploaded to a community album).
        :param crop_x: X coordinate of the left-upper corner
        :param crop_y: Y coordinate of the left-upper corner
        :param crop_x2: X coordinate of the right-bottom corner
        :param crop_y2: Y coordinate of the right-bottom corner
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = []
        method_name = 'photos.getOwnerCoverPhotoUploadServer'
        response_type = responses.BaseGetUploadServer
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def save_messages_photo(self, photo: str, server: Optional[int] = None, hash_: Optional[str] = None) -> responses.PhotosSaveMessagesPhoto:
        """
        Saves a photo after being successfully uploaded. URL obtained with [vk.com/dev/photos.getMessagesUploadServer|photos.getMessagesUploadServer] method.
        
        :param photo: Parameter returned when the photo is [vk.com/dev/upload_files|uploaded to the server].
        :param server:
        :param hash_:
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('hash_', 'hash')]
        method_name = 'photos.saveMessagesPhoto'
        response_type = responses.PhotosSaveMessagesPhoto
        return self._call(method_name, method_parameters, param_aliases, response_type)

    def save_owner_cover_photo(self, hash_: str, photo: str) -> responses.PhotosSaveOwnerCoverPhoto:
        """
        Saves cover photo after successful uploading.
        
        :param hash_: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """
    
        method_parameters = {k: v for k, v in locals().items() if k not in {'self', 'raw_response'}}
        param_aliases = [('hash_', 'hash')]
        method_name = 'photos.saveOwnerCoverPhoto'
        response_type = responses.PhotosSaveOwnerCoverPhoto
        return self._call(method_name, method_parameters, param_aliases, response_type)
